@@ -41,6 +41,7 @@
 #include <libgen.h>
 #include <linux/loop.h>
 #include <dirent.h>
+#include <sys/prctl.h>
 
 #include "lxc.h"
 #include "config.h"
@@ -54,14 +55,6 @@
 #include "lxclock.h"
 #include "lxc-btrfs.h"
 
-/* makedev() */
-#ifdef MAJOR_IN_MKDEV
-#    include <sys/mkdev.h>
-#endif
-#ifdef MAJOR_IN_SYSMACROS
-#    include <sys/sysmacros.h>
-#endif
-
 #ifndef BLKGETSIZE64
 #define BLKGETSIZE64 _IOR(0x12,114,size_t)
 #endif
@@ -70,20 +63,11 @@
 #define LO_FLAGS_AUTOCLEAR 4
 #endif
 
-#ifndef LOOP_CTL_GET_FREE
-#define LOOP_CTL_GET_FREE 0x4C82
-#endif
-
 #define DEFAULT_FS_SIZE 1073741824
 #define DEFAULT_FSTYPE "ext3"
 
 lxc_log_define(bdev, lxc);
 
-struct ovl_rsync_data {
-	struct bdev *orig;
-	struct bdev *new;
-};
-
 struct rsync_data_char {
 	char *src;
 	char *dest;
@@ -110,7 +94,7 @@ static int do_rsync(const char *src, const char *dest)
 	s[l-2] = '/';
 	s[l-1] = '\0';
 
-	execlp("rsync", "rsync", "-aHXS", "--delete", s, dest, (char *)NULL);
+	execlp("rsync", "rsync", "-a", s, dest, (char *)NULL);
 	exit(1);
 }
 
@@ -240,8 +224,12 @@ static int do_mkfs(const char *path, const char *fstype)
 
 	// If the file is not a block device, we don't want mkfs to ask
 	// us about whether to proceed.
-	if (null_stdfds() < 0)
-		exit(1);
+	close(0);
+	close(1);
+	close(2);
+	open("/dev/zero", O_RDONLY);
+	open("/dev/null", O_RDWR);
+	open("/dev/null", O_RDWR);
 	execlp("mkfs", "mkfs", "-t", fstype, path, NULL);
 	exit(1);
 }
@@ -369,15 +357,6 @@ struct bdev_type {
 	const struct bdev_ops *ops;
 };
 
-static int is_dir(const char *path)
-{
-	struct stat statbuf;
-	int ret = stat(path, &statbuf);
-	if (ret == 0 && S_ISDIR(statbuf.st_mode))
-		return 1;
-	return 0;
-}
-
 static int dir_detect(const char *path)
 {
 	if (strncmp(path, "dir:", 4) == 0)
@@ -495,7 +474,7 @@ static int dir_clonepaths(struct bdev *orig, struct bdev *new, const char *oldna
 
 static int dir_destroy(struct bdev *orig)
 {
-	if (lxc_rmdir_onedev(orig->src) < 0)
+	if (lxc_rmdir_onedev(orig->src, NULL) < 0)
 		return -1;
 	return 0;
 }
@@ -668,9 +647,6 @@ static int zfs_clone(const char *opath, const char *npath, const char *oname,
 		if ((pid = fork()) < 0)
 			return -1;
 		if (!pid) {
-			int dev0 = open("/dev/null", O_WRONLY);
-			if (dev0 >= 0)
-				dup2(dev0, STDERR_FILENO);
 			execlp("zfs", "zfs", "destroy", path1, NULL);
 			exit(1);
 		}
@@ -751,7 +727,7 @@ static int zfs_destroy(struct bdev *orig)
 		return -1;
 	*p = '\0';
 
-	execlp("zfs", "zfs", "destroy", "-r", output, NULL);
+	execlp("zfs", "zfs", "destroy", output, NULL);
 	exit(1);
 }
 
@@ -1277,7 +1253,7 @@ int btrfs_list_get_path_rootid(int fd, u64 *treeid)
 	return 0;
 }
 
-bool is_btrfs_fs(const char *path)
+static bool is_btrfs_fs(const char *path)
 {
 	int fd, ret;
 	struct btrfs_ioctl_space_args sargs;
@@ -1555,7 +1531,7 @@ static int btrfs_do_destroy_subvol(const char *path)
 
 	fd = open(newfull, O_RDONLY);
 	if (fd < 0) {
-		SYSERROR("Error opening %s", newfull);
+		ERROR("Error opening %s", newfull);
 		free(newfull);
 		return -1;
 	}
@@ -1731,7 +1707,7 @@ static int btrfs_recursive_destroy(const char *path)
 	struct btrfs_ioctl_search_header *sh;
 	struct btrfs_root_ref *ref;
 	struct my_btrfs_tree *tree;
-	int ret, e, i;
+	int ret, i;
 	unsigned long off = 0;
 	int name_len;
 	char *name;
@@ -1744,9 +1720,8 @@ static int btrfs_recursive_destroy(const char *path)
 	}
 
 	if (btrfs_list_get_path_rootid(fd, &root_id)) {
-		e = errno;
 		close(fd);
-		if (e == EPERM || e == EACCES) {
+		if (errno == EPERM || errno == EACCES) {
 			WARN("Will simply try removing");
 			goto ignore_search;
 		}
@@ -1777,16 +1752,10 @@ static int btrfs_recursive_destroy(const char *path)
 
 	while(1) {
 		ret = ioctl(fd, BTRFS_IOC_TREE_SEARCH, &args);
-		e = errno;
 		if (ret < 0) {
 			close(fd);
-			free_btrfs_tree(tree);
-			if (e == EPERM || e == EACCES) {
-				WARN("Warn: can't perform the search under %s. Will simply try removing", path);
-				goto ignore_search;
-			}
-
 			ERROR("Error: can't perform the search under %s\n", path);
+			free_btrfs_tree(tree);
 			return -1;
 		}
 		if (sk->nr_items == 0)
@@ -1860,13 +1829,6 @@ ignore_search:
 	return btrfs_do_destroy_subvol(path);
 }
 
-bool btrfs_try_remove_subvol(const char *path)
-{
-	if (!btrfs_detect(path))
-		return false;
-	return btrfs_recursive_destroy(path) == 0;
-}
-
 static int btrfs_destroy(struct bdev *orig)
 {
 	return btrfs_recursive_destroy(orig->src);
@@ -1903,9 +1865,9 @@ static int loop_detect(const char *path)
 	return 0;
 }
 
-static int find_free_loopdev_no_control(int *retfd, char *namep)
+static int find_free_loopdev(int *retfd, char *namep)
 {
-	struct dirent *direntp;
+	struct dirent dirent, *direntp;
 	struct loop_info64 lo;
 	DIR *dir;
 	int fd = -1;
@@ -1915,8 +1877,8 @@ static int find_free_loopdev_no_control(int *retfd, char *namep)
 		SYSERROR("Error opening /dev");
 		return -1;
 	}
+	while (!readdir_r(dir, &dirent, &direntp)) {
 
-	while ((direntp = readdir(dir))) {
 		if (!direntp)
 			break;
 		if (strncmp(direntp->d_name, "loop", 4) != 0)
@@ -1943,26 +1905,6 @@ static int find_free_loopdev_no_control(int *retfd, char *namep)
 	return 0;
 }
 
-static int find_free_loopdev(int *retfd, char *namep)
-{
-	int rc, fd = -1;
-	int ctl = open("/dev/loop-control", O_RDWR);
-	if (ctl < 0)
-		return find_free_loopdev_no_control(retfd, namep);
-	rc = ioctl(ctl, LOOP_CTL_GET_FREE);
-	if (rc >= 0) {
-		snprintf(namep, 100, "/dev/loop%d", rc);
-		fd = open(namep, O_RDWR);
-	}
-	close(ctl);
-	if (fd == -1) {
-		ERROR("No loop device found");
-		return -1;
-	}
-	*retfd = fd;
-	return 0;
-}
-
 static int loop_mount(struct bdev *bdev)
 {
 	int lfd, ffd = -1, ret = -1;
@@ -2250,10 +2192,6 @@ static int overlayfs_mount(struct bdev *bdev)
 	*upper = '\0';
 	upper++;
 
-	// if delta doesn't yet exist, create it
-	if (mkdir_p(upper, 0755) < 0 && errno != EEXIST)
-		return -22;
-
 	// overlayfs.v22 or higher needs workdir option
 	// if upper is /var/lib/lxc/c2/delta0,
 	// then workdir is /var/lib/lxc/c2/olwork
@@ -2272,11 +2210,6 @@ static int overlayfs_mount(struct bdev *bdev)
 		return -22;
 	}
 
-	if (mkdir_p(work, 0755) < 0 && errno != EEXIST) {
-		free(mntdata);
-		return -22;
-	}
-
 	// TODO We should check whether bdev->src is a blockdev, and if so
 	// but for now, only support overlays of a basic directory
 
@@ -2363,75 +2296,6 @@ static int rsync_delta_wrapper(void *data)
 	return rsync_delta(arg);
 }
 
-static int ovl_rsync(struct ovl_rsync_data *data)
-{
-	int ret;
-
-	if (setgid(0) < 0) {
-		ERROR("Failed to setgid to 0");
-		return -1;
-	}
-	if (setgroups(0, NULL) < 0)
-		WARN("Failed to clear groups");
-	if (setuid(0) < 0) {
-		ERROR("Failed to setuid to 0");
-		return -1;
-	}
-
-	if (unshare(CLONE_NEWNS) < 0) {
-		SYSERROR("Unable to unshare mounts ns");
-		return -1;
-	}
-	if (detect_shared_rootfs()) {
-		if (mount(NULL, "/", NULL, MS_SLAVE|MS_REC, NULL)) {
-			SYSERROR("Failed to make / rslave");
-			ERROR("Continuing...");
-		}
-	}
-	if (overlayfs_mount(data->orig) < 0) {
-		ERROR("Failed mounting original container fs");
-		return -1;
-	}
-	if (overlayfs_mount(data->new) < 0) {
-		ERROR("Failed mounting new container fs");
-		return -1;
-	}
-	ret = do_rsync(data->orig->dest, data->new->dest);
-
-	overlayfs_umount(data->new);
-	overlayfs_umount(data->orig);
-
-	if (ret < 0) {
-		ERROR("rsyncing %s to %s", data->orig->dest, data->new->dest);
-		return -1;
-	}
-
-	return 0;
-}
-
-static int ovl_rsync_wrapper(void *data)
-{
-	struct ovl_rsync_data *arg = data;
-	return ovl_rsync(arg);
-}
-
-static int ovl_do_rsync(struct bdev *orig, struct bdev *new, struct lxc_conf *conf)
-{
-	int ret = -1;
-	struct ovl_rsync_data rdata;
-
-	rdata.orig = orig;
-	rdata.new = new;
-	if (am_unpriv())
-		ret = userns_exec_1(conf, ovl_rsync_wrapper, &rdata);
-	else
-		ret = ovl_rsync(&rdata);
-	if (ret)
-		ERROR("copying overlayfs delta");
-
-	return ret;
-}
-
 static int overlayfs_clonepaths(struct bdev *orig, struct bdev *new, const char *oldname,
 		const char *cname, const char *oldpath, const char *lxcpath, int snap,
 		uint64_t newsize, struct lxc_conf *conf)
@@ -2487,15 +2351,12 @@ static int overlayfs_clonepaths(struct bdev *orig, struct bdev *new, const char
 		// and needs to be on the same filesystem as upperdir,
 		// so it's OK for it to be empty.
 		work = malloc(lastslashidx + 7);
-		if (!work) {
-			free(delta);
+		if (!work)
 			return -1;
-		}
 		strncpy(work, new->dest, lastslashidx+1);
 		strcpy(work+lastslashidx, "olwork");
 		if (mkdir(work, 0755) < 0) {
 			SYSERROR("error: mkdir %s", work);
-			free(delta);
 			free(work);
 			return -1;
 		}
@@ -2567,6 +2428,19 @@ static int overlayfs_clonepaths(struct bdev *orig, struct bdev *new, const char
 			WARN("Failed to update ownership of %s", work);
 		free(work);
 
+		struct rsync_data_char rdata;
+		rdata.src = odelta;
+		rdata.dest = ndelta;
+		if (am_unpriv())
+			ret = userns_exec_1(conf, rsync_delta_wrapper, &rdata);
+		else
+			ret = rsync_delta(&rdata);
+		if (ret) {
+			free(osrc);
+			free(ndelta);
+			ERROR("copying overlayfs delta");
+			return -1;
+		}
 		len = strlen(nsrc) + strlen(ndelta) + 12;
 		new->src = malloc(len);
 		if (!new->src) {
@@ -2579,8 +2453,6 @@ static int overlayfs_clonepaths(struct bdev *orig, struct bdev *new, const char
 		free(ndelta);
 		if (ret < 0 || ret >= len)
 			return -ENOMEM;
-
-		return ovl_do_rsync(orig, new, conf);
 	} else {
 		ERROR("overlayfs clone of %s container is not yet supported",
 			orig->type);
@@ -2602,7 +2474,7 @@ static int overlayfs_destroy(struct bdev *orig)
 	if (!upper)
 		return -22;
 	upper++;
-	return lxc_rmdir_onedev(upper);
+	return lxc_rmdir_onedev(upper, NULL);
 }
 
 /*
@@ -2680,12 +2552,12 @@ static int aufs_detect(const char *path)
 //
 static int aufs_mount(struct bdev *bdev)
 {
-	char *options, *dup, *lower, *upper;
+	char *options, *dup, *lower, *upper, *rundir;
 	int len;
 	unsigned long mntflags;
 	char *mntdata;
+	char *runpath;
 	int ret;
-	const char *xinopath = "/dev/shm/aufs.xino";
 
 	if (strcmp(bdev->type, "aufs"))
 		return -22;
@@ -2711,24 +2583,41 @@ static int aufs_mount(struct bdev *bdev)
 	// TODO We should check whether bdev->src is a blockdev, and if so
 	// but for now, only support aufs of a basic directory
 
+	rundir = get_rundir();
+	if (!rundir)
+		return -1;
+
+	len = strlen(rundir) + strlen("/lxc") + 1;
+	runpath = alloca(len);
+	ret = snprintf(runpath, len, "%s/lxc", rundir);
+	if (ret < 0 || ret >= len) {
+		free(mntdata);
+		free(rundir);
+		return -1;
+	}
+	if (mkdir_p(runpath, 0755) < 0) {
+		free(mntdata);
+		free(rundir);
+		return -1;
+	}
+
 	// AUFS does not work on top of certain filesystems like (XFS or Btrfs)
-	// so add xino=/dev/shm/aufs.xino parameter to mount options.
-	// The same xino option can be specified to multiple aufs mounts, and
-	// a xino file is not shared among multiple aufs mounts.
+	// so add xino=RUNDIR/lxc/aufs.xino parameter to mount options
 	//
 	// see http://www.mail-archive.com/aufs-users@lists.sourceforge.net/msg02587.html
-	//     http://www.mail-archive.com/aufs-users@lists.sourceforge.net/msg05126.html
 	if (mntdata) {
-		len = strlen(lower) + strlen(upper) + strlen(xinopath) + strlen("br==rw:=ro,,xino=") + strlen(mntdata) + 1;
+		len = strlen(lower) + strlen(upper) + strlen(runpath) + strlen("br==rw:=ro,,xino=/aufs.xino") + strlen(mntdata) + 1;
 		options = alloca(len);
-		ret = snprintf(options, len, "br=%s=rw:%s=ro,%s,xino=%s", upper, lower, mntdata, xinopath);
+		ret = snprintf(options, len, "br=%s=rw:%s=ro,%s,xino=%s/aufs.xino", upper, lower, mntdata, runpath);
 	}
 	else {
-		len = strlen(lower) + strlen(upper) + strlen(xinopath) + strlen("br==rw:=ro,xino=") + 1;
+		len = strlen(lower) + strlen(upper) + strlen(runpath) + strlen("br==rw:=ro,xino=/aufs.xino") + 1;
 		options = alloca(len);
-		ret = snprintf(options, len, "br=%s=rw:%s=ro,xino=%s", upper, lower, xinopath);
+		ret = snprintf(options, len, "br=%s=rw:%s=ro,xino=%s/aufs.xino", upper, lower, runpath);
 	}
 
+	free(rundir);
+
 	if (ret < 0 || ret >= len) {
 		free(mntdata);
 		return -1;
@@ -2771,9 +2660,6 @@ static int aufs_clonepaths(struct bdev *orig, struct bdev *new, const char *oldn
 	if (mkdir_p(new->dest, 0755) < 0)
 		return -1;
 
-	if (am_unpriv() && chown_mapped_root(new->dest, conf) < 0)
-		WARN("Failed to update ownership of %s", new->dest);
-
 	if (strcmp(orig->type, "dir") == 0) {
 		char *delta, *lastslash;
 		int ret, len, lastslashidx;
@@ -2798,8 +2684,6 @@ static int aufs_clonepaths(struct bdev *orig, struct bdev *new, const char *oldn
 			free(delta);
 			return -1;
 		}
-		if (am_unpriv() && chown_mapped_root(delta, conf) < 0)
-			WARN("Failed to update ownership of %s", delta);
 
 		// the src will be 'aufs:lowerdir:upperdir'
 		len = strlen(delta) + strlen(orig->src) + 12;
@@ -2833,23 +2717,7 @@ static int aufs_clonepaths(struct bdev *orig, struct bdev *new, const char *oldn
 			free(osrc);
 			return -ENOMEM;
 		}
-		if ((ret = mkdir(ndelta, 0755)) < 0 && errno != EEXIST) {
-			SYSERROR("error: mkdir %s", ndelta);
-			free(osrc);
-			free(ndelta);
-			return -1;
-		}
-		if (am_unpriv() && chown_mapped_root(ndelta, conf) < 0)
-			WARN("Failed to update ownership of %s", ndelta);
-
-		struct rsync_data_char rdata;
-		rdata.src = odelta;
-		rdata.dest = ndelta;
-		if (am_unpriv())
-			ret = userns_exec_1(conf, rsync_delta_wrapper, &rdata);
-		else
-			ret = rsync_delta(&rdata);
-		if (ret) {
+		if (do_rsync(odelta, ndelta) < 0) {
 			free(osrc);
 			free(ndelta);
 			ERROR("copying aufs delta");
@@ -2888,7 +2756,7 @@ static int aufs_destroy(struct bdev *orig)
 	if (!upper)
 		return -22;
 	upper++;
-	return lxc_rmdir_onedev(upper);
+	return lxc_rmdir_onedev(upper, NULL);
 }
 
 /*
@@ -2950,6 +2818,315 @@ static const struct bdev_ops aufs_ops = {
 	.can_backup = true,
 };
 
+//
+// nbd dev ops
+//
+
+static int nbd_detect(const char *path)
+{
+	if (strncmp(path, "nbd:", 4) == 0)
+		return 1;
+	return 0;
+}
+
+struct nbd_attach_data {
+	const char *nbd;
+	const char *path;
+};
+
+static void nbd_detach(const char *path)
+{
+	int ret;
+	pid_t pid = fork();
+
+	if (pid < 0) {
+		SYSERROR("Error forking to detach nbd");
+		return;
+	}
+	if (pid) {
+		ret = wait_for_pid(pid);
+		if (ret < 0)
+			ERROR("nbd disconnect returned an error");
+		return;
+	}
+	execlp("qemu-nbd", "qemu-nbd", "-d", path, NULL);
+	SYSERROR("Error executing qemu-nbd");
+	exit(1);
+}
+
+static int do_attach_nbd(void *d)
+{
+	struct nbd_attach_data *data = d;
+	const char *nbd, *path;
+	pid_t pid;
+	sigset_t mask;
+	int sfd;
+	ssize_t s;
+	struct signalfd_siginfo fdsi;
+
+	sigemptyset(&mask);
+	sigaddset(&mask, SIGHUP);
+	sigaddset(&mask, SIGCHLD);
+
+	nbd = data->nbd;
+	path = data->path;
+
+	if (sigprocmask(SIG_BLOCK, &mask, NULL) == -1) {
+		SYSERROR("Error blocking signals for nbd watcher");
+		exit(1);
+	}
+
+	sfd = signalfd(-1, &mask, 0);
+	if (sfd == -1) {
+		SYSERROR("Error opening signalfd for nbd task");
+		exit(1);
+	}
+
+	if (prctl(PR_SET_PDEATHSIG, SIGHUP, 0, 0, 0) < 0)
+		SYSERROR("Error setting parent death signal for nbd watcher");
+
+	pid = fork();
+	if (pid) {
+		for (;;) {
+			s = read(sfd, &fdsi, sizeof(struct signalfd_siginfo));
+			if (s != sizeof(struct signalfd_siginfo))
+				SYSERROR("Error reading from signalfd");
+
+			if (fdsi.ssi_signo == SIGHUP) {
+				/* container has exited */
+				nbd_detach(nbd);
+				exit(0);
+			} else if (fdsi.ssi_signo == SIGCHLD) {
+				int status;
+				/* If qemu-nbd fails, or is killed by a signal,
+				 * then exit */
+				while (waitpid(-1, &status, WNOHANG) > 0) {
+					if ((WIFEXITED(status) && WEXITSTATUS(status) != 0) ||
+							WIFSIGNALED(status)) {
+						nbd_detach(nbd);
+						exit(1);
+					}
+				}
+			}
+		}
+	}
+
+	close(sfd);
+	if (sigprocmask(SIG_UNBLOCK, &mask, NULL) == -1)
+		WARN("Warning: unblocking signals for nbd watcher");
+
+	execlp("qemu-nbd", "qemu-nbd", "-c", nbd, path, NULL);
+	SYSERROR("Error executing qemu-nbd");
+	exit(1);
+}
+
+static bool clone_attach_nbd(const char *nbd, const char *path)
+{
+	pid_t pid;
+	struct nbd_attach_data data;
+
+	data.nbd = nbd;
+	data.path = path;
+
+	pid = lxc_clone(do_attach_nbd, &data, CLONE_NEWPID);
+	if (pid < 0)
+		return false;
+	return true;
+}
+
+static bool nbd_busy(int idx)
+{
+	char path[100];
+	int ret;
+
+	ret = snprintf(path, 100, "/sys/block/nbd%d/pid", idx);
+	if (ret < 0 || ret >= 100)
+		return true;
+	return file_exists(path);
+}
+
+static bool attach_nbd(char *src, struct lxc_conf *conf)
+{
+	char *orig = alloca(strlen(src)+1), *p, path[50];
+	int i = 0;
+
+	strcpy(orig, src);
+	/* if path is followed by a partition, drop that for now */
+	p = strchr(orig, ':');
+	if (p)
+		*p = '\0';
+	while (1) {
+		sprintf(path, "/dev/nbd%d", i);
+		if (!file_exists(path))
+			return false;
+		if (nbd_busy(i)) {
+			i++;
+			continue;
+		}
+		if (!clone_attach_nbd(path, orig))
+			return false;
+		conf->nbd_idx = i;
+		return true;
+	}
+}
+
+static bool requires_nbd(const char *path)
+{
+	if (strncmp(path, "nbd:", 4) == 0)
+		return true;
+	return false;
+}
+
+/*
+ * attach_block_device returns true if all went well,
+ * meaning either a block device was attached or was not
+ * needed.  It returns false if something went wrong and
+ * container startup should be stopped.
+ */
+bool attach_block_device(struct lxc_conf *conf)
+{
+	char *path;
+
+	if (!conf->rootfs.path)
+		return true;
+	path = conf->rootfs.path;
+	if (!requires_nbd(path))
+		return true;
+	path = strchr(path, ':');
+	if (!path)
+		return false;
+	path++;
+	if (!attach_nbd(path, conf))
+		return false;
+	return true;
+}
+
+void detach_nbd_idx(int idx)
+{
+	int ret;
+	char path[50];
+
+	ret = snprintf(path, 50, "/dev/nbd%d", idx);
+	if (ret < 0 || ret >= 50)
+		return;
+
+	nbd_detach(path);
+}
+
+void detach_block_device(struct lxc_conf *conf)
+{
+	if (conf->nbd_idx != -1)
+		detach_nbd_idx(conf->nbd_idx);
+}
+
+/*
+ * Pick the partition # off the end of a nbd:file:p
+ * description.  Return 1-9 for the partition id, or 0
+ * for no partition.
+ */
+static int nbd_get_partition(const char *src)
+{
+	char *p = strchr(src, ':');
+	if (!p)
+		return 0;
+	p = strchr(p+1, ':');
+	if (!p)
+		return 0;
+	p++;
+	if (*p < '1' || *p > '9')
+		return 0;
+	return *p - '0';
+}
+
+static bool wait_for_partition(const char *path)
+{
+	int count = 0;
+	while (count < 5) {
+		if (file_exists(path))
+			return true;
+		sleep(1);
+		count++;
+	}
+	ERROR("Device %s did not show up after 5 seconds", path);
+	return false;
+}
+
+static int nbd_mount(struct bdev *bdev)
+{
+	int ret = -1, partition;
+	char path[50];
+
+	if (strcmp(bdev->type, "nbd"))
+		return -22;
+	if (!bdev->src || !bdev->dest)
+		return -22;
+
+	/* nbd_idx should have been copied by bdev_init from the lxc_conf */
+	if (bdev->nbd_idx < 0)
+		return -22;
+	partition = nbd_get_partition(bdev->src);
+	if (partition)
+		ret = snprintf(path, 50, "/dev/nbd%dp%d", bdev->nbd_idx,
+				partition);
+	else
+		ret = snprintf(path, 50, "/dev/nbd%d", bdev->nbd_idx);
+	if (ret < 0 || ret >= 50) {
+		ERROR("Error setting up nbd device path");
+		return ret;
+	}
+
+	/* It might take awhile for the partition files to show up */
+	if (partition) {
+		if (!wait_for_partition(path))
+			return -2;
+	}
+	ret = mount_unknown_fs(path, bdev->dest, bdev->mntopts);
+	if (ret < 0)
+		ERROR("Error mounting %s", bdev->src);
+
+	return ret;
+}
+
+static int nbd_create(struct bdev *bdev, const char *dest, const char *n,
+			struct bdev_specs *specs)
+{
+	return -ENOSYS;
+}
+
+static int nbd_clonepaths(struct bdev *orig, struct bdev *new, const char *oldname,
+		const char *cname, const char *oldpath, const char *lxcpath, int snap,
+		uint64_t newsize, struct lxc_conf *conf)
+{
+	return -ENOSYS;
+}
+
+static int nbd_destroy(struct bdev *orig)
+{
+	return -ENOSYS;
+}
+
+static int nbd_umount(struct bdev *bdev)
+{
+	int ret;
+
+	if (strcmp(bdev->type, "nbd"))
+		return -22;
+	if (!bdev->src || !bdev->dest)
+		return -22;
+	ret = umount(bdev->dest);
+	return ret;
+}
+
+static const struct bdev_ops nbd_ops = {
+	.detect = &nbd_detect,
+	.mount = &nbd_mount,
+	.umount = &nbd_umount,
+	.clone_paths = &nbd_clonepaths,
+	.destroy = &nbd_destroy,
+	.create = &nbd_create,
+	.can_snapshot = true,
+	.can_backup = false,
+};
 
 static const struct bdev_type bdevs[] = {
 	{.name = "zfs", .ops = &zfs_ops,},
@@ -2959,6 +3136,7 @@ static const struct bdev_type bdevs[] = {
 	{.name = "aufs", .ops = &aufs_ops,},
 	{.name = "overlayfs", .ops = &overlayfs_ops,},
 	{.name = "loop", .ops = &loop_ops,},
+	{.name = "nbd", .ops = &nbd_ops,},
 };
 
 static const size_t numbdevs = sizeof(bdevs) / sizeof(struct bdev_type);
@@ -3006,11 +3184,14 @@ static const struct bdev_type *bdev_query(const char *src)
 	return &bdevs[i];
 }
 
-struct bdev *bdev_init(const char *src, const char *dst, const char *mntopts)
+struct bdev *bdev_init(struct lxc_conf *conf, const char *src, const char *dst, const char *mntopts)
 {
 	struct bdev *bdev;
 	const struct bdev_type *q;
 
+	if (!src)
+		src = conf->rootfs.path;
+
 	if (!src)
 		return NULL;
 
@@ -3030,6 +3211,8 @@ struct bdev *bdev_init(const char *src, const char *dst, const char *mntopts)
 		bdev->src = strdup(src);
 	if (dst)
 		bdev->dest = strdup(dst);
+	if (strcmp(bdev->type, "nbd") == 0)
+		bdev->nbd_idx = conf->nbd_idx;
 
 	return bdev;
 }
@@ -3088,9 +3271,9 @@ static int rsync_rootfs_wrapper(void *data)
 	return rsync_rootfs(arg);
 }
 
-bool bdev_is_dir(const char *path)
+bool bdev_is_dir(struct lxc_conf *conf, const char *path)
 {
-	struct bdev *orig = bdev_init(path, NULL, NULL);
+	struct bdev *orig = bdev_init(conf, path, NULL, NULL);
 	bool ret = false;
 	if (!orig)
 		return ret;
@@ -3102,7 +3285,7 @@ bool bdev_is_dir(const char *path)
 
 bool bdev_can_backup(struct lxc_conf *conf)
 {
-	struct bdev *bdev = bdev_init(conf->rootfs.path, NULL, NULL);
+	struct bdev *bdev = bdev_init(conf, NULL, NULL, NULL);
 	bool ret;
 
 	if (!bdev)
@@ -3123,7 +3306,6 @@ static bool unpriv_snap_allowed(struct bdev *b, const char *t, bool snap,
 		// (unless snap && b->type == dir, in which case it will be
 		// overlayfs -- which is also allowed)
 		if (strcmp(b->type, "dir") == 0 ||
-				strcmp(b->type, "aufs") == 0 ||
 				strcmp(b->type, "overlayfs") == 0 ||
 				strcmp(b->type, "btrfs") == 0 ||
 				strcmp(b->type, "loop") == 0)
@@ -3133,11 +3315,8 @@ static bool unpriv_snap_allowed(struct bdev *b, const char *t, bool snap,
 
 	// unprivileged users can copy and snapshot dir, overlayfs,
 	// and loop.  In particular, not zfs, btrfs, or lvm.
-	if (strcmp(t, "dir") == 0 ||
-		strcmp(t, "aufs") == 0 ||
-		strcmp(t, "overlayfs") == 0 ||
-		strcmp(t, "btrfs") == 0 ||
-		strcmp(t, "loop") == 0)
+	if (strcmp(t, "dir") == 0 || strcmp(t, "overlayfs") == 0 ||
+			strcmp(t, "btrfs") == 0 || strcmp(t, "loop") == 0)
 		return true;
 	return false;
 }
@@ -3171,7 +3350,7 @@ struct bdev *bdev_copy(struct lxc_container *c0, const char *cname,
 		return NULL;
 	}
 
-	orig = bdev_init(src, NULL, NULL);
+	orig = bdev_init(c0->lxc_conf, src, NULL, NULL);
 	if (!orig) {
 		ERROR("failed to detect blockdev type for %s", src);
 		return NULL;
