@@ -39,76 +39,6 @@ struct execute_args {
 	int quiet;
 };
 
-/* historically lxc-init has been under /usr/lib/lxc and under
- * /usr/lib/$ARCH/lxc.  It now lives as $prefix/sbin/init.lxc.
- */
-static char *choose_init(void)
-{
-	char *retv = NULL;
-	int ret, env_set = 0;
-	struct stat mystat;
-
-	if (!getenv("PATH")) {
-		if (setenv("PATH", "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin", 0))
-			SYSERROR("Failed to setenv");
-		env_set = 1;
-	}
-
-	retv = on_path("init.lxc");
-
-	if (env_set) {
-		if (unsetenv("PATH"))
-			SYSERROR("Failed to unsetenv");
-	}
-
-	if (retv)
-		return retv;
-
-	retv = malloc(PATH_MAX);
-	if (!retv)
-		return NULL;
-
-	ret = snprintf(retv, PATH_MAX, SBINDIR "/init.lxc");
-	if (ret < 0 || ret >= PATH_MAX) {
-		ERROR("pathname too long");
-		goto out1;
-	}
-
-	ret = stat(retv, &mystat);
-	if (ret == 0)
-		return retv;
-
-	ret = snprintf(retv, PATH_MAX, LXCINITDIR "/lxc/lxc-init");
-	if (ret < 0 || ret >= PATH_MAX) {
-		ERROR("pathname too long");
-		goto out1;
-	}
-
-	ret = stat(retv, &mystat);
-	if (ret == 0)
-		return retv;
-
-	ret = snprintf(retv, PATH_MAX, "/usr/lib/lxc/lxc-init");
-	if (ret < 0 || ret >= PATH_MAX) {
-		ERROR("pathname too long");
-		goto out1;
-	}
-	ret = stat(retv, &mystat);
-	if (ret == 0)
-		return retv;
-	ret = snprintf(retv, PATH_MAX, "/sbin/lxc-init");
-	if (ret < 0 || ret >= PATH_MAX) {
-		ERROR("pathname too long");
-		goto out1;
-	}
-	ret = stat(retv, &mystat);
-	if (ret == 0)
-		return retv;
-out1:
-	free(retv);
-	return NULL;
-}
-
 static int execute_start(struct lxc_handler *handler, void* data)
 {
 	int j, i = 0;
@@ -132,9 +62,9 @@ static int execute_start(struct lxc_handler *handler, void* data)
 	if (!argv)
 		goto out1;
 
-	initpath = choose_init();
+	initpath = choose_init(NULL);
 	if (!initpath) {
-		ERROR("Failed to find an lxc-init or init.lxc");
+		ERROR("Failed to find an lxc-init");
 		goto out2;
 	}
 	argv[i++] = initpath;
@@ -188,7 +118,7 @@ int lxc_execute(const char *name, char *const argv[], int quiet,
 		.quiet = quiet
 	};
 
-	if (lxc_check_inherited(conf, -1))
+	if (lxc_check_inherited(conf, false, -1))
 		return -1;
 
 	conf->is_execute = 1;
