@@ -27,6 +27,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
+#include <pthread.h>
 
 #define __USE_GNU /* for *_CLOEXEC */
 
@@ -40,27 +41,44 @@
 #define LXC_LOG_PREFIX_SIZE	32
 #define LXC_LOG_BUFFER_SIZE	512
 
-#ifdef HAVE_TLS
-__thread int lxc_log_fd = -1;
-static __thread char log_prefix[LXC_LOG_PREFIX_SIZE] = "lxc";
-static __thread char *log_fname = NULL;
-/* command line values for logfile or logpriority should always override
- * values from the configuration file or defaults
- */
-static __thread int lxc_logfile_specified = 0;
-static __thread int lxc_loglevel_specified = 0;
-static __thread int lxc_quiet_specified = 0;
-#else
+static pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;
+static void lock_mutex(pthread_mutex_t *l)
+{
+	int ret;
+
+	if ((ret = pthread_mutex_lock(l)) != 0) {
+		fprintf(stderr, "pthread_mutex_lock returned:%d %s\n", ret, strerror(ret));
+		exit(1);
+	}
+}
+
+static void unlock_mutex(pthread_mutex_t *l)
+{
+	int ret;
+
+	if ((ret = pthread_mutex_unlock(l)) != 0) {
+		fprintf(stderr, "pthread_mutex_unlock returned:%d %s\n", ret, strerror(ret));
+		exit(1);
+	}
+}
+
+void log_lock(void)
+{
+	lock_mutex(&log_mutex);
+}
+
+void log_unlock(void)
+{
+	unlock_mutex(&log_mutex);
+}
+
 int lxc_log_fd = -1;
+int lxc_quiet_specified;
+int lxc_log_use_global_fd;
+static int lxc_loglevel_specified;
+
 static char log_prefix[LXC_LOG_PREFIX_SIZE] = "lxc";
 static char *log_fname = NULL;
-static int lxc_quiet_specified = 0;
-/* command line values for logfile or logpriority should always override
- * values from the configuration file or defaults
- */
-static int lxc_logfile_specified = 0;
-static int lxc_loglevel_specified = 0;
-#endif
 
 lxc_log_define(lxc_log, lxc);
 
@@ -85,8 +103,17 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 	char buffer[LXC_LOG_BUFFER_SIZE];
 	int n;
 	int ms;
+	int fd_to_use = -1;
 
-	if (lxc_log_fd == -1)
+#ifndef NO_LXC_CONF
+	if (!lxc_log_use_global_fd && current_config)
+		fd_to_use = current_config->logfd;
+#endif
+
+	if (fd_to_use == -1)
+		fd_to_use = lxc_log_fd;
+
+	if (fd_to_use == -1)
 		return 0;
 
 	ms = event->timestamp.tv_usec / 1000;
@@ -111,7 +138,7 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 
 	buffer[n] = '\n';
 
-	return write(lxc_log_fd, buffer, n + 1);
+	return write(fd_to_use, buffer, n + 1);
 }
 
 static struct lxc_log_appender log_appender_stderr = {
@@ -136,7 +163,7 @@ static struct lxc_log_category log_root = {
 struct lxc_log_category lxc_log_category_lxc = {
 	.name		= "lxc",
 	.priority	= LXC_LOG_PRIORITY_ERROR,
-	.appender	= &log_appender_stderr,
+	.appender	= &log_appender_logfile,
 	.parent		= &log_root
 };
 
@@ -305,6 +332,11 @@ static int _lxc_log_set_file(const char *name, const char *lxcpath, int create_d
 	return ret;
 }
 
+/*
+ * lxc_log_init:
+ * Called from lxc front-end programs (like lxc-create, lxc-start) to
+ * initalize the log defaults.
+ */
 extern int lxc_log_init(const char *name, const char *file,
 			const char *priority, const char *prefix, int quiet,
 			const char *lxcpath)
@@ -320,10 +352,12 @@ extern int lxc_log_init(const char *name, const char *file,
 	if (priority)
 		lxc_priority = lxc_log_priority_to_int(priority);
 
-	lxc_log_category_lxc.priority = lxc_priority;
+	if (!lxc_loglevel_specified) {
+		lxc_log_category_lxc.priority = lxc_priority;
+		lxc_loglevel_specified = 1;
+	}
 
 	if (!lxc_quiet_specified) {
-		lxc_log_category_lxc.appender = &log_appender_logfile;
 		if (!quiet)
 			lxc_log_category_lxc.appender->next = &log_appender_stderr;
 	}
@@ -335,6 +369,7 @@ extern int lxc_log_init(const char *name, const char *file,
 		if (strcmp(file, "none") == 0)
 			return 0;
 		ret = __lxc_log_set_file(file, 1);
+		lxc_log_use_global_fd = 1;
 	} else {
 		/* if no name was specified, there nothing to do */
 		if (!name)
@@ -385,15 +420,13 @@ extern void lxc_log_close(void)
  * happens after processing command line arguments, which override the .conf
  * settings.  So only set the level if previously unset.
  */
-extern int lxc_log_set_level(int level)
+extern int lxc_log_set_level(int *dest, int level)
 {
-	if (lxc_loglevel_specified)
-		return 0;
 	if (level < 0 || level >= LXC_LOG_PRIORITY_NOTSET) {
 		ERROR("invalid log priority %d", level);
 		return -1;
 	}
-	lxc_log_category_lxc.priority = level;
+	*dest = level;
 	return 0;
 }
 
@@ -415,11 +448,23 @@ extern bool lxc_log_has_valid_level(void)
  * happens after processing command line arguments, which override the .conf
  * settings.  So only set the file if previously unset.
  */
-extern int lxc_log_set_file(const char *fname)
+extern int lxc_log_set_file(int *fd, const char *fname)
 {
-	if (lxc_logfile_specified)
-		return 0;
-	return __lxc_log_set_file(fname, 0);
+	if (*fd != -1) {
+		close(*fd);
+		*fd = -1;
+	}
+
+	if (build_dir(fname)) {
+		ERROR("failed to create dir for log file \"%s\" : %s", fname,
+				strerror(errno));
+		return -1;
+	}
+
+	*fd = log_open(fname);
+	if (*fd == -1)
+		return -errno;
+	return 0;
 }
 
 extern const char *lxc_log_get_file(void)
@@ -440,11 +485,6 @@ extern const char *lxc_log_get_prefix(void)
 
 extern void lxc_log_options_no_override()
 {
-	if (lxc_log_get_file())
-		lxc_logfile_specified = 1;
-
-	if (lxc_log_get_level() != LXC_LOG_PRIORITY_NOTSET)
-		lxc_loglevel_specified = 1;
-
 	lxc_quiet_specified = 1;
+	lxc_loglevel_specified = 1;
 }
