@@ -40,6 +40,7 @@ static int my_parser(struct lxc_arguments* args, int c, char* arg)
 	case 'r': args->reboot = 1; break;
 	case 's': args->shutdown = 1; break;
 	case 'a': args->all = 1; break;
+	case 'A': args->ignore_auto = 1; break;
 	case 'g': cmd_groups_list = accumulate_list( arg, ",", cmd_groups_list); break;
 	case 't': args->timeout = atoi(arg); break;
 	}
@@ -52,6 +53,7 @@ static const struct option my_longopts[] = {
 	{"reboot", no_argument, 0, 'r'},
 	{"shutdown", no_argument, 0, 's'},
 	{"all", no_argument, 0, 'a'},
+	{"ignore-auto", no_argument, 0, 'A'},
 	{"groups", required_argument, 0, 'g'},
 	{"timeout", required_argument, 0, 't'},
 	{"help", no_argument, 0, 'h'},
@@ -71,6 +73,7 @@ Options:\n\
   -s, --shutdown    shutdown the containers instead of starting them\n\
 \n\
   -a, --all         list all auto-started containers (ignore groups)\n\
+  -A, --ignore-auto ignore lxc.start.auto and select all matching containers\n\
   -g, --groups      list of groups (comma separated) to select\n\
   -t, --timeout=T   wait T seconds before hard-stopping\n",
 	.options  = my_longopts,
@@ -174,10 +177,8 @@ static struct lxc_list *accumulate_list(char *input, char *delimiter, struct lxc
 		if ( list_contains_entry( workptr, workstr_list ) ) {
 			if ( *workptr ) {
 				fprintf(stderr, "Duplicate group \"%s\" in list - ignoring\n", workptr );
-				fflush(stderr);
 			} else {
-				fprintf(stderr, "Duplicate NULL group in list - ignoring\n" );
-				fflush(stderr);
+				fprintf(stderr, "Duilicate NULL group in list - ignoring\n" );
 			}
 		} else {
 			worklist = malloc(sizeof(*worklist));
@@ -329,10 +330,6 @@ int main(int argc, char *argv[])
 	struct lxc_container **containers = NULL;
 	struct lxc_list **c_groups_lists = NULL;
 	struct lxc_list *cmd_group;
-	char *const default_start_args[] = {
-		"/sbin/init",
-		NULL,
-	};
 
 	if (lxc_arguments_parse(&my_args, argc, argv))
 		return 1;
@@ -355,8 +352,7 @@ int main(int argc, char *argv[])
 	qsort(&containers[0], count, sizeof(struct lxc_container *), cmporder);
 
 	if (cmd_groups_list && my_args.all) {
-		fprintf(stderr, "Specifying -a (all) with -g (groups) doesn't make sense. All option overrides.\n");
-		fflush(stderr);
+		fprintf(stderr, "Specifying -a (all) with -g (groups) doesn't make sense. All option overrides.");
 	}
 
 	if (!cmd_groups_list) {
@@ -397,7 +393,8 @@ int main(int argc, char *argv[])
 				continue;
 			}
 
-			if (get_config_integer(c, "lxc.start.auto") != 1) {
+			if (!my_args.ignore_auto &&
+			    get_config_integer(c, "lxc.start.auto") != 1) {
 				/* We're done with this container */
 				if ( lxc_container_put(c) > 0 )
 					containers[i] = NULL;
@@ -426,15 +423,12 @@ int main(int argc, char *argv[])
 			if (my_args.shutdown) {
 				/* Shutdown the container */
 				if (c->is_running(c)) {
-					if (my_args.list) {
+					if (my_args.list)
 						printf("%s\n", c->name);
-						fflush(stdout);
-					}
 					else {
 						if (!c->shutdown(c, my_args.timeout)) {
 							if (!c->stop(c)) {
 								fprintf(stderr, "Error shutting down container: %s\n", c->name);
-								fflush(stderr);
 							}
 						}
 					}
@@ -443,31 +437,23 @@ int main(int argc, char *argv[])
 			else if (my_args.hardstop) {
 				/* Kill the container */
 				if (c->is_running(c)) {
-					if (my_args.list) {
+					if (my_args.list)
 						printf("%s\n", c->name);
-						fflush(stdout);
-					}
 					else {
-						if (!c->stop(c)) {
+						if (!c->stop(c))
 							fprintf(stderr, "Error killing container: %s\n", c->name);
-							fflush(stderr);
-						}
 					}
 				}
 			}
 			else if (my_args.reboot) {
 				/* Reboot the container */
 				if (c->is_running(c)) {
-					if (my_args.list) {
+					if (my_args.list)
 						printf("%s %d\n", c->name,
-							get_config_integer(c, "lxc.start.delay"));
-						fflush(stdout);
-					}
+						       get_config_integer(c, "lxc.start.delay"));
 					else {
-						if (!c->reboot(c)) {
+						if (!c->reboot(c))
 							fprintf(stderr, "Error rebooting container: %s\n", c->name);
-							fflush(stderr);
-						}
 						else
 							sleep(get_config_integer(c, "lxc.start.delay"));
 					}
@@ -476,16 +462,12 @@ int main(int argc, char *argv[])
 			else {
 				/* Start the container */
 				if (!c->is_running(c)) {
-					if (my_args.list) {
+					if (my_args.list)
 						printf("%s %d\n", c->name,
-							get_config_integer(c, "lxc.start.delay"));
-						fflush(stdout);
-					}
+						       get_config_integer(c, "lxc.start.delay"));
 					else {
-						if (!c->start(c, 0, default_start_args)) {
+						if (!c->start(c, 0, NULL))
 							fprintf(stderr, "Error starting container: %s\n", c->name);
-							fflush(stderr);
-						}
 						else
 							sleep(get_config_integer(c, "lxc.start.delay"));
 					}
@@ -505,6 +487,7 @@ int main(int argc, char *argv[])
 				c_groups_lists[i] = NULL;
 			}
 		}
+
 	}
 
 	/* clean up any lingering detritus */
